import { NgZone } from 'angular2/core';
import { Platform } from '../../platform/platform';
/**
 * @private
 */
export declare class DisplayWhen {
    protected isMatch: boolean;
    private platform;
    private conditions;
    constructor(conditions: string, platform: Platform, ngZone: NgZone);
    orientation(): boolean;
}
/**
 *
 * @name ShowWhen
 * @description
 * The `showWhen` attribute takes a string that represents a plaform or screen orientation.
 * The element the attribute is added to will only be shown when that platform or screen orientation is active.
 * Complements the [hideWhen attribute](../HideWhen).
 * @usage
 * ```html
 * <div showWhen="ios">I am only visible on iOS!</div>
 * ```
 * @demo /docs/v2/demos/show-when/
 * @see {@link ../HideWhen HideWhen API Docs}
 */
export declare class ShowWhen extends DisplayWhen {
    constructor(showWhen: string, platform: Platform, ngZone: NgZone);
    /**
     * @private
     */
    hidden: boolean;
}
/**
 * @name HideWhen
 * @description
 * The `hideWhen` attribute takes a string that represents a plaform or screen orientation.
 * The element the attribute is added to will only be hidden when that platform or screen orientation is active.
 * Complements the [showWhen attribute](../ShowWhen).
 * @usage
 * ```html
 * <div hideWhen="android">I am hidden on Android!</div>
 * ```
 * @demo /docs/v2/demos/hide-when/
 * @see {@link ../ShowWhen ShowWhen API Docs}
 */
export declare class HideWhen extends DisplayWhen {
    constructor(hideWhen: string, platform: Platform, ngZone: NgZone);
    /**
     * @private
     */
    hidden: boolean;
}
